@@ -3,12 +3,6 @@ package docker
 import (
 	"bytes"
 	"fmt"
-	"github.com/dotcloud/docker"
-	"github.com/dotcloud/docker/engine"
-	"github.com/dotcloud/docker/nat"
-	"github.com/dotcloud/docker/runconfig"
-	"github.com/dotcloud/docker/sysinit"
-	"github.com/dotcloud/docker/utils"
 	"io"
 	"log"
 	"net"
@@ -21,37 +15,50 @@ import (
 	"syscall"
 	"testing"
 	"time"
+
+	"github.com/dotcloud/docker/daemon"
+	"github.com/dotcloud/docker/engine"
+	"github.com/dotcloud/docker/image"
+	"github.com/dotcloud/docker/nat"
+	"github.com/dotcloud/docker/runconfig"
+	"github.com/dotcloud/docker/sysinit"
+	"github.com/dotcloud/docker/utils"
 )
 
 const (
-	unitTestImageName     = "docker-test-image"
-	unitTestImageID       = "83599e29c455eb719f77d799bc7c51521b9551972f5a850d7ad265bc1b5292f6" // 1.0
-	unitTestImageIDShort  = "83599e29c455"
-	unitTestNetworkBridge = "testdockbr0"
-	unitTestStoreBase     = "/var/lib/docker/unit-tests"
-	testDaemonAddr        = "127.0.0.1:4270"
-	testDaemonProto       = "tcp"
+	unitTestImageName        = "docker-test-image"
+	unitTestImageID          = "83599e29c455eb719f77d799bc7c51521b9551972f5a850d7ad265bc1b5292f6" // 1.0
+	unitTestImageIDShort     = "83599e29c455"
+	unitTestNetworkBridge    = "testdockbr0"
+	unitTestStoreBase        = "/var/lib/docker/unit-tests"
+	testDaemonAddr           = "127.0.0.1:4270"
+	testDaemonProto          = "tcp"
+	testDaemonHttpsProto     = "tcp"
+	testDaemonHttpsAddr      = "localhost:4271"
+	testDaemonRogueHttpsAddr = "localhost:4272"
 )
 
 var (
-	// FIXME: globalRuntime is deprecated by globalEngine. All tests should be converted.
-	globalRuntime   *docker.Runtime
-	globalEngine    *engine.Engine
-	startFds        int
-	startGoroutines int
+	// FIXME: globalDaemon is deprecated by globalEngine. All tests should be converted.
+	globalDaemon           *daemon.Daemon
+	globalEngine           *engine.Engine
+	globalHttpsEngine      *engine.Engine
+	globalRogueHttpsEngine *engine.Engine
+	startFds               int
+	startGoroutines        int
 )
 
-// FIXME: nuke() is deprecated by Runtime.Nuke()
-func nuke(runtime *docker.Runtime) error {
-	return runtime.Nuke()
+// FIXME: nuke() is deprecated by Daemon.Nuke()
+func nuke(daemon *daemon.Daemon) error {
+	return daemon.Nuke()
 }
 
 // FIXME: cleanup and nuke are redundant.
 func cleanup(eng *engine.Engine, t *testing.T) error {
-	runtime := mkRuntimeFromEngine(eng, t)
-	for _, container := range runtime.List() {
+	daemon := mkDaemonFromEngine(eng, t)
+	for _, container := range daemon.List() {
 		container.Kill()
-		runtime.Destroy(container)
+		daemon.Destroy(container)
 	}
 	job := eng.Job("images")
 	images, err := job.Stdout.AddTable()
@@ -113,21 +120,23 @@ func init() {
 		src.Close()
 	}
 
-	// Setup the base runtime, which will be duplicated for each test.
+	// Setup the base daemon, which will be duplicated for each test.
 	// (no tests are run directly in the base)
 	setupBaseImage()
 
-	// Create the "global runtime" with a long-running daemon for integration tests
+	// Create the "global daemon" with a long-running daemons for integration tests
 	spawnGlobalDaemon()
+	spawnLegitHttpsDaemon()
+	spawnRogueHttpsDaemon()
 	startFds, startGoroutines = utils.GetTotalUsedFds(), runtime.NumGoroutine()
 }
 
 func setupBaseImage() {
 	eng := newTestEngine(log.New(os.Stderr, "", 0), false, unitTestStoreBase)
-	job := eng.Job("inspect", unitTestImageName, "image")
+	job := eng.Job("image_inspect", unitTestImageName)
 	img, _ := job.Stdout.AddEnv()
 	// If the unit test is not found, try to download it.
-	if err := job.Run(); err != nil || img.Get("id") != unitTestImageID {
+	if err := job.Run(); err != nil || img.Get("Id") != unitTestImageID {
 		// Retrieve the Image
 		job = eng.Job("pull", unitTestImageName)
 		job.Stdout.Add(utils.NopWriteCloser(os.Stdout))
@@ -138,14 +147,14 @@ func setupBaseImage() {
 }
 
 func spawnGlobalDaemon() {
-	if globalRuntime != nil {
-		utils.Debugf("Global runtime already exists. Skipping.")
+	if globalDaemon != nil {
+		utils.Debugf("Global daemon already exists. Skipping.")
 		return
 	}
 	t := log.New(os.Stderr, "", 0)
 	eng := NewTestEngine(t)
 	globalEngine = eng
-	globalRuntime = mkRuntimeFromEngine(eng, t)
+	globalDaemon = mkDaemonFromEngine(eng, t)
 
 	// Spawn a Daemon
 	go func() {
@@ -170,10 +179,65 @@ func spawnGlobalDaemon() {
 	}
 }
 
+func spawnLegitHttpsDaemon() {
+	if globalHttpsEngine != nil {
+		return
+	}
+	globalHttpsEngine = spawnHttpsDaemon(testDaemonHttpsAddr, "fixtures/https/ca.pem",
+		"fixtures/https/server-cert.pem", "fixtures/https/server-key.pem")
+}
+
+func spawnRogueHttpsDaemon() {
+	if globalRogueHttpsEngine != nil {
+		return
+	}
+	globalRogueHttpsEngine = spawnHttpsDaemon(testDaemonRogueHttpsAddr, "fixtures/https/ca.pem",
+		"fixtures/https/server-rogue-cert.pem", "fixtures/https/server-rogue-key.pem")
+}
+
+func spawnHttpsDaemon(addr, cacert, cert, key string) *engine.Engine {
+	t := log.New(os.Stderr, "", 0)
+	root, err := newTestDirectory(unitTestStoreBase)
+	if err != nil {
+		t.Fatal(err)
+	}
+	// FIXME: here we don't use NewTestEngine because it calls initserver with Autorestart=false,
+	// and we want to set it to true.
+
+	eng := newTestEngine(t, true, root)
+
+	// Spawn a Daemon
+	go func() {
+		utils.Debugf("Spawning https daemon for integration tests")
+		listenURL := &url.URL{
+			Scheme: testDaemonHttpsProto,
+			Host:   addr,
+		}
+		job := eng.Job("serveapi", listenURL.String())
+		job.SetenvBool("Logging", true)
+		job.SetenvBool("Tls", true)
+		job.SetenvBool("TlsVerify", true)
+		job.Setenv("TlsCa", cacert)
+		job.Setenv("TlsCert", cert)
+		job.Setenv("TlsKey", key)
+		if err := job.Run(); err != nil {
+			log.Fatalf("Unable to spawn the test daemon: %s", err)
+		}
+	}()
+
+	// Give some time to ListenAndServer to actually start
+	time.Sleep(time.Second)
+
+	if err := eng.Job("acceptconnections").Run(); err != nil {
+		log.Fatalf("Unable to accept connections for test api: %s", err)
+	}
+	return eng
+}
+
 // FIXME: test that ImagePull(json=true) send correct json output
 
-func GetTestImage(runtime *docker.Runtime) *docker.Image {
-	imgs, err := runtime.Graph().Map()
+func GetTestImage(daemon *daemon.Daemon) *image.Image {
+	imgs, err := daemon.Graph().Map()
 	if err != nil {
 		log.Fatalf("Unable to get the test image: %s", err)
 	}
@@ -182,21 +246,21 @@ func GetTestImage(runtime *docker.Runtime) *docker.Image {
 			return image
 		}
 	}
-	log.Fatalf("Test image %v not found in %s: %s", unitTestImageID, runtime.Graph().Root, imgs)
+	log.Fatalf("Test image %v not found in %s: %s", unitTestImageID, daemon.Graph().Root, imgs)
 	return nil
 }
 
-func TestRuntimeCreate(t *testing.T) {
-	runtime := mkRuntime(t)
-	defer nuke(runtime)
+func TestDaemonCreate(t *testing.T) {
+	daemon := mkDaemon(t)
+	defer nuke(daemon)
 
 	// Make sure we start we 0 containers
-	if len(runtime.List()) != 0 {
-		t.Errorf("Expected 0 containers, %v found", len(runtime.List()))
+	if len(daemon.List()) != 0 {
+		t.Errorf("Expected 0 containers, %v found", len(daemon.List()))
 	}
 
-	container, _, err := runtime.Create(&runconfig.Config{
-		Image: GetTestImage(runtime).ID,
+	container, _, err := daemon.Create(&runconfig.Config{
+		Image: GetTestImage(daemon).ID,
 		Cmd:   []string{"ls", "-al"},
 	},
 		"",
@@ -206,56 +270,56 @@ func TestRuntimeCreate(t *testing.T) {
 	}
 
 	defer func() {
-		if err := runtime.Destroy(container); err != nil {
+		if err := daemon.Destroy(container); err != nil {
 			t.Error(err)
 		}
 	}()
 
 	// Make sure we can find the newly created container with List()
-	if len(runtime.List()) != 1 {
-		t.Errorf("Expected 1 container, %v found", len(runtime.List()))
+	if len(daemon.List()) != 1 {
+		t.Errorf("Expected 1 container, %v found", len(daemon.List()))
 	}
 
 	// Make sure the container List() returns is the right one
-	if runtime.List()[0].ID != container.ID {
-		t.Errorf("Unexpected container %v returned by List", runtime.List()[0])
+	if daemon.List()[0].ID != container.ID {
+		t.Errorf("Unexpected container %v returned by List", daemon.List()[0])
 	}
 
 	// Make sure we can get the container with Get()
-	if runtime.Get(container.ID) == nil {
+	if daemon.Get(container.ID) == nil {
 		t.Errorf("Unable to get newly created container")
 	}
 
 	// Make sure it is the right container
-	if runtime.Get(container.ID) != container {
+	if daemon.Get(container.ID) != container {
 		t.Errorf("Get() returned the wrong container")
 	}
 
 	// Make sure Exists returns it as existing
-	if !runtime.Exists(container.ID) {
+	if !daemon.Exists(container.ID) {
 		t.Errorf("Exists() returned false for a newly created container")
 	}
 
 	// Test that conflict error displays correct details
-	testContainer, _, _ := runtime.Create(
+	testContainer, _, _ := daemon.Create(
 		&runconfig.Config{
-			Image: GetTestImage(runtime).ID,
+			Image: GetTestImage(daemon).ID,
 			Cmd:   []string{"ls", "-al"},
 		},
 		"conflictname",
 	)
-	if _, _, err := runtime.Create(&runconfig.Config{Image: GetTestImage(runtime).ID, Cmd: []string{"ls", "-al"}}, testContainer.Name); err == nil || !strings.Contains(err.Error(), utils.TruncateID(testContainer.ID)) {
+	if _, _, err := daemon.Create(&runconfig.Config{Image: GetTestImage(daemon).ID, Cmd: []string{"ls", "-al"}}, testContainer.Name); err == nil || !strings.Contains(err.Error(), utils.TruncateID(testContainer.ID)) {
 		t.Fatalf("Name conflict error doesn't include the correct short id. Message was: %s", err.Error())
 	}
 
 	// Make sure create with bad parameters returns an error
-	if _, _, err = runtime.Create(&runconfig.Config{Image: GetTestImage(runtime).ID}, ""); err == nil {
+	if _, _, err = daemon.Create(&runconfig.Config{Image: GetTestImage(daemon).ID}, ""); err == nil {
 		t.Fatal("Builder.Create should throw an error when Cmd is missing")
 	}
 
-	if _, _, err := runtime.Create(
+	if _, _, err := daemon.Create(
 		&runconfig.Config{
-			Image: GetTestImage(runtime).ID,
+			Image: GetTestImage(daemon).ID,
 			Cmd:   []string{},
 		},
 		"",
@@ -264,20 +328,20 @@ func TestRuntimeCreate(t *testing.T) {
 	}
 
 	config := &runconfig.Config{
-		Image:     GetTestImage(runtime).ID,
+		Image:     GetTestImage(daemon).ID,
 		Cmd:       []string{"/bin/ls"},
 		PortSpecs: []string{"80"},
 	}
-	container, _, err = runtime.Create(config, "")
+	container, _, err = daemon.Create(config, "")
 
-	_, err = runtime.Commit(container, "testrepo", "testtag", "", "", config)
+	_, err = daemon.Commit(container, "testrepo", "testtag", "", "", config)
 	if err != nil {
 		t.Error(err)
 	}
 
 	// test expose 80:8000
-	container, warnings, err := runtime.Create(&runconfig.Config{
-		Image:     GetTestImage(runtime).ID,
+	container, warnings, err := daemon.Create(&runconfig.Config{
+		Image:     GetTestImage(daemon).ID,
 		Cmd:       []string{"ls", "-al"},
 		PortSpecs: []string{"80:8000"},
 	},
@@ -292,83 +356,84 @@ func TestRuntimeCreate(t *testing.T) {
 }
 
 func TestDestroy(t *testing.T) {
-	runtime := mkRuntime(t)
-	defer nuke(runtime)
+	daemon := mkDaemon(t)
+	defer nuke(daemon)
 
-	container, _, err := runtime.Create(&runconfig.Config{
-		Image: GetTestImage(runtime).ID,
+	container, _, err := daemon.Create(&runconfig.Config{
+		Image: GetTestImage(daemon).ID,
 		Cmd:   []string{"ls", "-al"},
 	}, "")
 	if err != nil {
 		t.Fatal(err)
 	}
 	// Destroy
-	if err := runtime.Destroy(container); err != nil {
+	if err := daemon.Destroy(container); err != nil {
 		t.Error(err)
 	}
 
-	// Make sure runtime.Exists() behaves correctly
-	if runtime.Exists("test_destroy") {
+	// Make sure daemon.Exists() behaves correctly
+	if daemon.Exists("test_destroy") {
 		t.Errorf("Exists() returned true")
 	}
 
-	// Make sure runtime.List() doesn't list the destroyed container
-	if len(runtime.List()) != 0 {
-		t.Errorf("Expected 0 container, %v found", len(runtime.List()))
+	// Make sure daemon.List() doesn't list the destroyed container
+	if len(daemon.List()) != 0 {
+		t.Errorf("Expected 0 container, %v found", len(daemon.List()))
 	}
 
-	// Make sure runtime.Get() refuses to return the unexisting container
-	if runtime.Get(container.ID) != nil {
+	// Make sure daemon.Get() refuses to return the unexisting container
+	if daemon.Get(container.ID) != nil {
 		t.Errorf("Unable to get newly created container")
 	}
 
 	// Test double destroy
-	if err := runtime.Destroy(container); err == nil {
+	if err := daemon.Destroy(container); err == nil {
 		// It should have failed
 		t.Errorf("Double destroy did not fail")
 	}
 }
 
 func TestGet(t *testing.T) {
-	runtime := mkRuntime(t)
-	defer nuke(runtime)
+	daemon := mkDaemon(t)
+	defer nuke(daemon)
 
-	container1, _, _ := mkContainer(runtime, []string{"_", "ls", "-al"}, t)
-	defer runtime.Destroy(container1)
+	container1, _, _ := mkContainer(daemon, []string{"_", "ls", "-al"}, t)
+	defer daemon.Destroy(container1)
 
-	container2, _, _ := mkContainer(runtime, []string{"_", "ls", "-al"}, t)
-	defer runtime.Destroy(container2)
+	container2, _, _ := mkContainer(daemon, []string{"_", "ls", "-al"}, t)
+	defer daemon.Destroy(container2)
 
-	container3, _, _ := mkContainer(runtime, []string{"_", "ls", "-al"}, t)
-	defer runtime.Destroy(container3)
+	container3, _, _ := mkContainer(daemon, []string{"_", "ls", "-al"}, t)
+	defer daemon.Destroy(container3)
 
-	if runtime.Get(container1.ID) != container1 {
-		t.Errorf("Get(test1) returned %v while expecting %v", runtime.Get(container1.ID), container1)
+	if daemon.Get(container1.ID) != container1 {
+		t.Errorf("Get(test1) returned %v while expecting %v", daemon.Get(container1.ID), container1)
 	}
 
-	if runtime.Get(container2.ID) != container2 {
-		t.Errorf("Get(test2) returned %v while expecting %v", runtime.Get(container2.ID), container2)
+	if daemon.Get(container2.ID) != container2 {
+		t.Errorf("Get(test2) returned %v while expecting %v", daemon.Get(container2.ID), container2)
 	}
 
-	if runtime.Get(container3.ID) != container3 {
-		t.Errorf("Get(test3) returned %v while expecting %v", runtime.Get(container3.ID), container3)
+	if daemon.Get(container3.ID) != container3 {
+		t.Errorf("Get(test3) returned %v while expecting %v", daemon.Get(container3.ID), container3)
 	}
 
 }
 
-func startEchoServerContainer(t *testing.T, proto string) (*docker.Runtime, *docker.Container, string) {
+func startEchoServerContainer(t *testing.T, proto string) (*daemon.Daemon, *daemon.Container, string) {
 	var (
-		err     error
-		id      string
-		strPort string
-		eng     = NewTestEngine(t)
-		runtime = mkRuntimeFromEngine(eng, t)
-		port    = 5554
-		p       nat.Port
+		err          error
+		id           string
+		outputBuffer = bytes.NewBuffer(nil)
+		strPort      string
+		eng          = NewTestEngine(t)
+		daemon       = mkDaemonFromEngine(eng, t)
+		port         = 5554
+		p            nat.Port
 	)
 	defer func() {
 		if err != nil {
-			runtime.Nuke()
+			daemon.Nuke()
 		}
 	}()
 
@@ -392,11 +457,12 @@ func startEchoServerContainer(t *testing.T, proto string) (*docker.Runtime, *doc
 		jobCreate.SetenvList("Cmd", []string{"sh", "-c", cmd})
 		jobCreate.SetenvList("PortSpecs", []string{fmt.Sprintf("%s/%s", strPort, proto)})
 		jobCreate.SetenvJson("ExposedPorts", ep)
-		jobCreate.Stdout.AddString(&id)
+		jobCreate.Stdout.Add(outputBuffer)
 		if err := jobCreate.Run(); err != nil {
 			t.Fatal(err)
 		}
-		// FIXME: this relies on the undocumented behavior of runtime.Create
+		id = engine.Tail(outputBuffer, 1)
+		// FIXME: this relies on the undocumented behavior of daemon.Create
 		// which will return a nil error AND container if the exposed ports
 		// are invalid. That behavior should be fixed!
 		if id != "" {
@@ -418,7 +484,7 @@ func startEchoServerContainer(t *testing.T, proto string) (*docker.Runtime, *doc
 		t.Fatal(err)
 	}
 
-	container := runtime.Get(id)
+	container := daemon.Get(id)
 	if container == nil {
 		t.Fatalf("Couldn't fetch test container %s", id)
 	}
@@ -433,13 +499,13 @@ func startEchoServerContainer(t *testing.T, proto string) (*docker.Runtime, *doc
 	container.WaitTimeout(500 * time.Millisecond)
 
 	strPort = container.NetworkSettings.Ports[p][0].HostPort
-	return runtime, container, strPort
+	return daemon, container, strPort
 }
 
 // Run a container with a TCP port allocated, and test that it can receive connections on localhost
 func TestAllocateTCPPortLocalhost(t *testing.T) {
-	runtime, container, port := startEchoServerContainer(t, "tcp")
-	defer nuke(runtime)
+	daemon, container, port := startEchoServerContainer(t, "tcp")
+	defer nuke(daemon)
 	defer container.Kill()
 
 	for i := 0; i != 10; i++ {
@@ -487,8 +553,8 @@ func TestAllocateTCPPortLocalhost(t *testing.T) {
 
 // Run a container with an UDP port allocated, and test that it can receive connections on localhost
 func TestAllocateUDPPortLocalhost(t *testing.T) {
-	runtime, container, port := startEchoServerContainer(t, "udp")
-	defer nuke(runtime)
+	daemon, container, port := startEchoServerContainer(t, "udp")
+	defer nuke(daemon)
 	defer container.Kill()
 
 	conn, err := net.Dial("udp", fmt.Sprintf("localhost:%v", port))
@@ -523,15 +589,15 @@ func TestAllocateUDPPortLocalhost(t *testing.T) {
 
 func TestRestore(t *testing.T) {
 	eng := NewTestEngine(t)
-	runtime1 := mkRuntimeFromEngine(eng, t)
-	defer runtime1.Nuke()
+	daemon1 := mkDaemonFromEngine(eng, t)
+	defer daemon1.Nuke()
 	// Create a container with one instance of docker
-	container1, _, _ := mkContainer(runtime1, []string{"_", "ls", "-al"}, t)
-	defer runtime1.Destroy(container1)
+	container1, _, _ := mkContainer(daemon1, []string{"_", "ls", "-al"}, t)
+	defer daemon1.Destroy(container1)
 
 	// Create a second container meant to be killed
-	container2, _, _ := mkContainer(runtime1, []string{"-i", "_", "/bin/cat"}, t)
-	defer runtime1.Destroy(container2)
+	container2, _, _ := mkContainer(daemon1, []string{"-i", "_", "/bin/cat"}, t)
+	defer daemon1.Destroy(container2)
 
 	// Start the container non blocking
 	if err := container2.Start(); err != nil {
@@ -551,8 +617,8 @@ func TestRestore(t *testing.T) {
 	container2.State.SetRunning(42)
 	container2.ToDisk()
 
-	if len(runtime1.List()) != 2 {
-		t.Errorf("Expected 2 container, %v found", len(runtime1.List()))
+	if len(daemon1.List()) != 2 {
+		t.Errorf("Expected 2 container, %v found", len(daemon1.List()))
 	}
 	if err := container1.Run(); err != nil {
 		t.Fatal(err)
@@ -564,13 +630,13 @@ func TestRestore(t *testing.T) {
 
 	// Here are are simulating a docker restart - that is, reloading all containers
 	// from scratch
-	eng = newTestEngine(t, false, eng.Root())
-	runtime2 := mkRuntimeFromEngine(eng, t)
-	if len(runtime2.List()) != 2 {
-		t.Errorf("Expected 2 container, %v found", len(runtime2.List()))
+	eng = newTestEngine(t, false, daemon1.Config().Root)
+	daemon2 := mkDaemonFromEngine(eng, t)
+	if len(daemon2.List()) != 2 {
+		t.Errorf("Expected 2 container, %v found", len(daemon2.List()))
 	}
 	runningCount := 0
-	for _, c := range runtime2.List() {
+	for _, c := range daemon2.List() {
 		if c.State.IsRunning() {
 			t.Errorf("Running container found: %v (%v)", c.ID, c.Path)
 			runningCount++
@@ -579,7 +645,7 @@ func TestRestore(t *testing.T) {
 	if runningCount != 0 {
 		t.Fatalf("Expected 0 container alive, %d found", runningCount)
 	}
-	container3 := runtime2.Get(container1.ID)
+	container3 := daemon2.Get(container1.ID)
 	if container3 == nil {
 		t.Fatal("Unable to Get container")
 	}
@@ -591,22 +657,22 @@ func TestRestore(t *testing.T) {
 
 func TestDefaultContainerName(t *testing.T) {
 	eng := NewTestEngine(t)
-	runtime := mkRuntimeFromEngine(eng, t)
-	defer nuke(runtime)
+	daemon := mkDaemonFromEngine(eng, t)
+	defer nuke(daemon)
 
 	config, _, _, err := runconfig.Parse([]string{unitTestImageID, "echo test"}, nil)
 	if err != nil {
 		t.Fatal(err)
 	}
 
-	container := runtime.Get(createNamedTestContainer(eng, config, t, "some_name"))
+	container := daemon.Get(createNamedTestContainer(eng, config, t, "some_name"))
 	containerID := container.ID
 
 	if container.Name != "/some_name" {
 		t.Fatalf("Expect /some_name got %s", container.Name)
 	}
 
-	if c := runtime.Get("/some_name"); c == nil {
+	if c := daemon.Get("/some_name"); c == nil {
 		t.Fatalf("Couldn't retrieve test container as /some_name")
 	} else if c.ID != containerID {
 		t.Fatalf("Container /some_name has ID %s instead of %s", c.ID, containerID)
@@ -615,22 +681,22 @@ func TestDefaultContainerName(t *testing.T) {
 
 func TestRandomContainerName(t *testing.T) {
 	eng := NewTestEngine(t)
-	runtime := mkRuntimeFromEngine(eng, t)
-	defer nuke(runtime)
+	daemon := mkDaemonFromEngine(eng, t)
+	defer nuke(daemon)
 
-	config, _, _, err := runconfig.Parse([]string{GetTestImage(runtime).ID, "echo test"}, nil)
+	config, _, _, err := runconfig.Parse([]string{GetTestImage(daemon).ID, "echo test"}, nil)
 	if err != nil {
 		t.Fatal(err)
 	}
 
-	container := runtime.Get(createTestContainer(eng, config, t))
+	container := daemon.Get(createTestContainer(eng, config, t))
 	containerID := container.ID
 
 	if container.Name == "" {
 		t.Fatalf("Expected not empty container name")
 	}
 
-	if c := runtime.Get(container.Name); c == nil {
+	if c := daemon.Get(container.Name); c == nil {
 		log.Fatalf("Could not lookup container %s by its name", container.Name)
 	} else if c.ID != containerID {
 		log.Fatalf("Looking up container name %s returned id %s instead of %s", container.Name, c.ID, containerID)
@@ -639,8 +705,8 @@ func TestRandomContainerName(t *testing.T) {
 
 func TestContainerNameValidation(t *testing.T) {
 	eng := NewTestEngine(t)
-	runtime := mkRuntimeFromEngine(eng, t)
-	defer nuke(runtime)
+	daemon := mkDaemonFromEngine(eng, t)
+	defer nuke(daemon)
 
 	for _, test := range []struct {
 		Name  string
@@ -657,12 +723,12 @@ func TestContainerNameValidation(t *testing.T) {
 			t.Fatal(err)
 		}
 
-		var shortID string
+		var outputBuffer = bytes.NewBuffer(nil)
 		job := eng.Job("create", test.Name)
 		if err := job.ImportEnv(config); err != nil {
 			t.Fatal(err)
 		}
-		job.Stdout.AddString(&shortID)
+		job.Stdout.Add(outputBuffer)
 		if err := job.Run(); err != nil {
 			if !test.Valid {
 				continue
@@ -670,13 +736,13 @@ func TestContainerNameValidation(t *testing.T) {
 			t.Fatal(err)
 		}
 
-		container := runtime.Get(shortID)
+		container := daemon.Get(engine.Tail(outputBuffer, 1))
 
 		if container.Name != "/"+test.Name {
 			t.Fatalf("Expect /%s got %s", test.Name, container.Name)
 		}
 
-		if c := runtime.Get("/" + test.Name); c == nil {
+		if c := daemon.Get("/" + test.Name); c == nil {
 			t.Fatalf("Couldn't retrieve test container as /%s", test.Name)
 		} else if c.ID != container.ID {
 			t.Fatalf("Container /%s has ID %s instead of %s", test.Name, c.ID, container.ID)
@@ -687,17 +753,17 @@ func TestContainerNameValidation(t *testing.T) {
 
 func TestLinkChildContainer(t *testing.T) {
 	eng := NewTestEngine(t)
-	runtime := mkRuntimeFromEngine(eng, t)
-	defer nuke(runtime)
+	daemon := mkDaemonFromEngine(eng, t)
+	defer nuke(daemon)
 
 	config, _, _, err := runconfig.Parse([]string{unitTestImageID, "echo test"}, nil)
 	if err != nil {
 		t.Fatal(err)
 	}
 
-	container := runtime.Get(createNamedTestContainer(eng, config, t, "/webapp"))
+	container := daemon.Get(createNamedTestContainer(eng, config, t, "/webapp"))
 
-	webapp, err := runtime.GetByName("/webapp")
+	webapp, err := daemon.GetByName("/webapp")
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -706,19 +772,19 @@ func TestLinkChildContainer(t *testing.T) {
 		t.Fatalf("Expect webapp id to match container id: %s != %s", webapp.ID, container.ID)
 	}
 
-	config, _, _, err = runconfig.Parse([]string{GetTestImage(runtime).ID, "echo test"}, nil)
+	config, _, _, err = runconfig.Parse([]string{GetTestImage(daemon).ID, "echo test"}, nil)
 	if err != nil {
 		t.Fatal(err)
 	}
 
-	childContainer := runtime.Get(createTestContainer(eng, config, t))
+	childContainer := daemon.Get(createTestContainer(eng, config, t))
 
-	if err := runtime.RegisterLink(webapp, childContainer, "db"); err != nil {
+	if err := daemon.RegisterLink(webapp, childContainer, "db"); err != nil {
 		t.Fatal(err)
 	}
 
 	// Get the child by it's new name
-	db, err := runtime.GetByName("/webapp/db")
+	db, err := daemon.GetByName("/webapp/db")
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -729,17 +795,17 @@ func TestLinkChildContainer(t *testing.T) {
 
 func TestGetAllChildren(t *testing.T) {
 	eng := NewTestEngine(t)
-	runtime := mkRuntimeFromEngine(eng, t)
-	defer nuke(runtime)
+	daemon := mkDaemonFromEngine(eng, t)
+	defer nuke(daemon)
 
 	config, _, _, err := runconfig.Parse([]string{unitTestImageID, "echo test"}, nil)
 	if err != nil {
 		t.Fatal(err)
 	}
 
-	container := runtime.Get(createNamedTestContainer(eng, config, t, "/webapp"))
+	container := daemon.Get(createNamedTestContainer(eng, config, t, "/webapp"))
 
-	webapp, err := runtime.GetByName("/webapp")
+	webapp, err := daemon.GetByName("/webapp")
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -753,13 +819,13 @@ func TestGetAllChildren(t *testing.T) {
 		t.Fatal(err)
 	}
 
-	childContainer := runtime.Get(createTestContainer(eng, config, t))
+	childContainer := daemon.Get(createTestContainer(eng, config, t))
 
-	if err := runtime.RegisterLink(webapp, childContainer, "db"); err != nil {
+	if err := daemon.RegisterLink(webapp, childContainer, "db"); err != nil {
 		t.Fatal(err)
 	}
 
-	children, err := runtime.Children("/webapp")
+	children, err := daemon.Children("/webapp")
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -782,11 +848,11 @@ func TestGetAllChildren(t *testing.T) {
 }
 
 func TestDestroyWithInitLayer(t *testing.T) {
-	runtime := mkRuntime(t)
-	defer nuke(runtime)
+	daemon := mkDaemon(t)
+	defer nuke(daemon)
 
-	container, _, err := runtime.Create(&runconfig.Config{
-		Image: GetTestImage(runtime).ID,
+	container, _, err := daemon.Create(&runconfig.Config{
+		Image: GetTestImage(daemon).ID,
 		Cmd:   []string{"ls", "-al"},
 	}, "")
 
@@ -794,29 +860,29 @@ func TestDestroyWithInitLayer(t *testing.T) {
 		t.Fatal(err)
 	}
 	// Destroy
-	if err := runtime.Destroy(container); err != nil {
+	if err := daemon.Destroy(container); err != nil {
 		t.Fatal(err)
 	}
 
-	// Make sure runtime.Exists() behaves correctly
-	if runtime.Exists("test_destroy") {
+	// Make sure daemon.Exists() behaves correctly
+	if daemon.Exists("test_destroy") {
 		t.Fatalf("Exists() returned true")
 	}
 
-	// Make sure runtime.List() doesn't list the destroyed container
-	if len(runtime.List()) != 0 {
-		t.Fatalf("Expected 0 container, %v found", len(runtime.List()))
+	// Make sure daemon.List() doesn't list the destroyed container
+	if len(daemon.List()) != 0 {
+		t.Fatalf("Expected 0 container, %v found", len(daemon.List()))
 	}
 
-	driver := runtime.Graph().Driver()
+	driver := daemon.Graph().Driver()
 
 	// Make sure that the container does not exist in the driver
-	if _, err := driver.Get(container.ID); err == nil {
+	if _, err := driver.Get(container.ID, ""); err == nil {
 		t.Fatal("Conttainer should not exist in the driver")
 	}
 
 	// Make sure that the init layer is removed from the driver
-	if _, err := driver.Get(fmt.Sprintf("%s-init", container.ID)); err == nil {
+	if _, err := driver.Get(fmt.Sprintf("%s-init", container.ID), ""); err == nil {
 		t.Fatal("Container's init layer should not exist in the driver")
 	}
 }
